#!/bin/bash -e
node_status_handler() {
    # echo "An error occurred: $1"
    # Perform any necessary actions to handle the error

    # Make the curl call to report the error
    timestamp=$(date '+%s')
    consoleID=$(uuidgen)
    curl --location --request POST "$PIPELINES_SHARED_ARTIFACTORY_BASEURLUI/pipelines/api/v1/nodeConsoles" \
    --header "Authorization: Bearer $PIPELINES_NODE_API_TOKEN" \
    --header 'Content-Type: application/json' \
    --data "{
        \"nodeConsoleModels\": [
            {
                \"nodeId\": \"$NODE_ID\",
                \"consoleId\": \"$consoleID\",
                \"parentConsoleId\": \"root\",
                \"timestamp\": \"$timestamp\",
                \"projectId\": \"$PROJECT_ID\",
                \"type\": \"msg\",
                \"message\": \"$1\",
                \"timestampEndedAt\": null,
                \"isSuccess\": false,
                \"isShown\": null
            }
        ]
    }"
}

check_buildplane_url_status() {
  local url=$1

  node_status_handler "Checking the URL $url"
  response_code=$(curl -H "Authorization: Bearer $PIPELINES_NODE_API_TOKEN" -I "$url" $CURL_OPTS -o /dev/null -w "%{http_code}" -s -L)
  
  if [ "$response_code" != "200" ]; then
    node_status_handler "Unable to connect to $url"
    echo "Response code: $response_code, exiting"
    exit 1
  fi

}

download_buildplane_file() {
  local url=$1
  local filename=$2

  node_status_handler "Attempting to download $url"
  if ! curl -H "Authorization: Bearer $PIPELINES_NODE_API_TOKEN" "$url" $CURL_OPTS --output "$filename"; then
    node_status_handler "Failed to download $url"
    exit 1
  fi

  node_status_handler "Downloaded $url"
}

update_build_node_status() {
  node_status_handler "Node init script ran successfully"
  init_time=$(date -u +"%FT%TZ")
  api_response_code=$(curl -s $CURL_OPTS -X PUT -H "Authorization: Bearer $PIPELINES_NODE_API_TOKEN" -H "Content-Type: application/json" -d '{"lastInitializedAt":"'$init_time'"}' $PIPELINES_CORE_SERVICES_API_EXTERNALURL"/nodes/"$NODE_ID -o -l -i -w "%{http_code}")
  
  node_status_handler "node status $api_response_code"
  
  if [ "$api_response_code" != "200" ]; then
    node_status_handler "Node status failed"
    echo "Response code: $api_response_code, exiting"
    exit 1
  fi
}
# Exporting env variables required for booting reqKick inside EC2 instance
# NOTE: this file cannot be more than 16KB
export JFROG_DIR="/var/opt/jfrog/pipelines"
export REQKICK_DIR="$JFROG_DIR/reqKick"
export EXECTEMPLATES_DIR="$REQKICK_DIR/node_modules/@jfrog-pipelines/core/execTemplates"
export EXTENSIONS_DIR="$JFROG_DIR/extensions"
export REQEXEC_DIR="$JFROG_DIR/reqExec"
export REPORTS_DIR="$JFROG_DIR/reports"
export NODEINIT_DIR="$JFROG_DIR/nodeInit"
export REQKICK_CONFIG_DIR="$JFROG_DIR/var/etc/reqKick"
export REQKICK_SERVICE_NAME="pipelines-reqKick"
export CUSTOM_CERTS_DIR="$JFROG_DIR/var/etc/security/keys/trusted"
export CUSTOM_CERTS_FILE_LOC="/etc/ssl/certs/ca-bundle.crt"
export NODE_OPERATING_SYSTEM="<%= operatingSystem %>"
export NODE_ARCHITECTURE="<%= architecture %>"
export RUN_MODE="<%= runMode %>"
export NODE_ID="<%= nodeId %>"
export PROJECT_ID="<%= projectId %>"
export BUILD_PLANE_VERSION="<%= buildPlaneVersion %>"
export BUILDPLANE_RPM_DOWNLOAD_URL="<%= buildPlaneArtifactDownloadUrl %>"
export PIPELINES_SHARED_ARTIFACTORY_BASEURLUI="<%= baseUrlUI %>"
export PIPELINES_CORE_SERVICES_WWW_EXTERNALURL="<%= wwwUrl %>"
export PIPELINES_CORE_SERVICES_API_EXTERNALURL="<%= apiExternalUrl %>"
export CONFIGURE_NODE=false
export ENABLE_SWAP=false
export BOOT_BUILD_AGENT=true
export INSECURE_DOCKER_REGISTRIES='<%= insecureDockerRegistries %>'
export PIPELINES_NODE_API_TOKEN='<%= token %>'
export NODE_POLLER_INTERVAL_MS=<%= nodePollerIntervalMS %>
export CUSTOM_CERTS_DATA=<%= customCertsData %>
export CUSTOM_CERTS_ENABLED=<%= customCertsEnabled %>
export NO_VERIFY_SSL=<%= noVerifySsl %>

CURL_OPTS=""
if [ "$CUSTOM_CERTS_ENABLED" == true ]; then
  mkdir -p $CUSTOM_CERTS_DIR
  echo -e $CUSTOM_CERTS_DATA > $CUSTOM_CERTS_DIR/ca.crt
  cp $CUSTOM_CERTS_DIR/ca.crt /usr/share/pki/ca-trust-source/anchors/
  update-ca-trust extract
elif [ "$NO_VERIFY_SSL" == true ]; then
  CURL_OPTS="--insecure"
fi
 
check_buildplane_url_status "$BUILDPLANE_RPM_DOWNLOAD_URL"

download_buildplane_file "$BUILDPLANE_RPM_DOWNLOAD_URL" "buildPlane.rpm"

rpm -i --prefix=$JFROG_DIR --force buildPlane.rpm

source $NODEINIT_DIR/$NODE_ARCHITECTURE/$NODE_OPERATING_SYSTEM/boot.sh

update_build_node_status